@@ -43,9 +43,15 @@ The following functions are available for X-ray scatting information processing:
      :func:`xray_sld`
          Computes xray scattering length densities for molecules.
 
+     :func:`index_of_refraction`
+         Express xray scattering length density as an index of refraction
+
+     :func:`mirror_reflectivity`
+         X-ray reflectivity from a mirror made of a single compound.
+
      :func:`xray_sld_from_atoms`
          The underlying scattering length density calculator. This works with
-         a dictionary of atoms and quanties directly.
+         a dictionary of atoms and quantities directly.
 
      :func:`emission_table`
          Prints a table of emission lines.
@@ -69,88 +75,87 @@ X-ray f1 and f2 tables
 ======================
 The data for the tables is stored in the ``periodictable/xsf``.
 directory.  The following information is from ``periodictable/xsf/read.me``,
-with minor formatting changes.
-These ``[*.nff]`` files were used to generate the tables published in
-reference [#Henke1993]_. The files contain three columns of data:
+with minor formatting changes:
+
+  These ``[*.nff]`` files were used to generate the tables published in
+  reference [#Henke1993]_. The files contain three columns of data:
 
     Energy(eV), *f_1*, *f_2*,
 
-where *f_1* and *f_2* are the atomic (forward) scattering factors.
-There are 500+ points on a uniform logarithmic mesh with points
-added 0.1 eV above and below "sharp" absorption edges. The
-tabulated values of *f_1* contain a relativistic, energy
-independent, correction given by:
+  where *f_1* and *f_2* are the atomic (forward) scattering factors.
+  There are 500+ points on a uniform logarithmic mesh with points
+  added 0.1 eV above and below "sharp" absorption edges. The
+  tabulated values of *f_1* contain a relativistic, energy
+  independent, correction given by:
 
-.. math::
+  .. math::
 
     Z^* = Z - (Z/82.5)^{2.37}
 
-.. Note::
+  .. Note::
     Below 29 eV *f_1* is set equal to -9999.
 
-The atomic photoabsorption cross section, $\mu_a$, may be readily
-obtained from the values of $f_2$ using the relation:
+  The atomic photoabsorption cross section, $\mu_a$, may be readily
+  obtained from the values of $f_2$ using the relation:
 
-.. math::
+  .. math::
 
     \mu_a = 2 r_e \lambda f_2
 
-where $r_e$ is the classical electron radius, and $\lambda$ is
-the wavelength. The index of refraction for a material with *N* atoms per
-unit volume is calculated by:
+  where $r_e$ is the classical electron radius, and $\lambda$ is
+  the wavelength. The index of refraction for a material with *N* atoms per
+  unit volume is calculated by:
 
-.. math::
+  .. math::
 
     n = 1 - N r_e \lambda^2 (f_1 + i f_2)/(2 \pi).
 
-These (semi-empirical) atomic scattering factors are based upon
-photoabsorption measurements of elements in their elemental state.
-The basic assumption is that condensed matter may be modeled as a
-collection of non-interacting atoms.  This assumption is in general
-a good one for energies sufficiently far from absorption thresholds.
-In the threshold regions, the specific chemical state is important
-and direct experimental measurements must be made.
+  These (semi-empirical) atomic scattering factors are based upon
+  photoabsorption measurements of elements in their elemental state.
+  The basic assumption is that condensed matter may be modeled as a
+  collection of non-interacting atoms.  This assumption is in general
+  a good one for energies sufficiently far from absorption thresholds.
+  In the threshold regions, the specific chemical state is important
+  and direct experimental measurements must be made.
 
-These tables are based on a compilation of the available experimental
-measurements and theoretical calculations.  For many elements there is
-little or no published data and in such cases it was necessary to
-rely on theoretical calculations and interpolations across Z.
-In order to improve the accuracy in the future considerably more
-experimental measurements are needed.
+  These tables are based on a compilation of the available experimental
+  measurements and theoretical calculations.  For many elements there is
+  little or no published data and in such cases it was necessary to
+  rely on theoretical calculations and interpolations across Z.
+  In order to improve the accuracy in the future considerably more
+  experimental measurements are needed.
 
-Please send any comments about the tables to EMGullikson@lbl.gov.
+  Note that the following elements have been updated since the
+  publication of Ref. [#Henke1993]_ in July 1993. Check
+  `<http://henke.lbl.gov/optical_constants/update.html>`_ for more
+  recent updates.
+
+  .. table::
+
+           ========  ==========   =================
+           Element   Updated      Energy Range (eV)
+           ========  ==========   =================
+           Mg        Jan 2011     10-1300
+           Zr        Apr 2010     20-1000
+           La        Jun 2007     14-440
+           Gd        Jun 2007     12-450
+           Sc        Apr 2006     50-1300
+           Ti        Aug 2004     20-150
+           Ru        Aug 2004     40-1300
+           W         Aug 2004     35-250
+           Mo        Aug 2004     25-60
+           Be        Aug 2004     40-250
+           Mo        Nov 1997     10-930
+           Fe        Oct 1995     600-800
+           Si        Jun 1995     30-500
+           Au        Jul 1994     2000-6500
+           Mg,Al,Si  Jan 1994     30-200
+           Li        Nov 1994     2000-30000
+           ========  ==========   =================
 
 
-Note that the following elements have been updated since the
-publication of Ref. [#Henke1993]_ in July 1993.
-
-.. table::
-
-           ========  ==========   ==============
-           Element   Updated      Energy Range
-           ========  ==========   ==============
-           Mg        1/15/94      30-50 eV
-           Al        1/15/94      30-73 eV
-           Si        1/15/94      30-100 eV
-           Au        11/7/94      2000-6500 eV
-           Li        11/15/94     2000-30000 eV
-           Si        6/95         30-500 eV
-           Fe        10/95        600-800 eV
-           Mo        11/97        10-930 eV
-           Be        8/04         40-250 eV
-           Mo        8/04         25-60 eV
-           W         8/04         35-250 eV
-           Ru        8/04         40-1300 eV
-           Ti        8/04         20-150 eV
-           Sc        4/06         50-1300 eV
-           Gd        6/07         12-450 eV
-           La        6/07         14-440 eV
-           ========  ==========   ==============
-
-
-Data available at:
-  #. http://henke.lbl.gov/optical_constants/asf.html
-  #. http://henke.lbl.gov/optical_constants/update.html
+  Data available at:
+    #. http://henke.lbl.gov/optical_constants/asf.html
 
 .. [#Henke1993] B. L. Henke, E. M. Gullikson, and J. C. Davis.  "X-ray interactions:
        photoabsorption, scattering, transmission, and reflection at E=50-30000 eV,
@@ -162,12 +167,13 @@ __all__ = ['Xray', 'init', 'init_spectral_lines',
            'xray_energy','xray_wavelength',
            'xray_sld','xray_sld_from_atoms',
            'emission_table','sld_table','plot_xsf',
+           'index_of_refraction', 'mirror_reflectivity',
            ]
 import os.path
 import glob
 
 import numpy
-from numpy import nan
+from numpy import nan, pi, exp, sin, cos, sqrt, radians
 
 from .core import Element, Ion, default_table, get_data_path
 from .constants import (avogadro_number, plancks_constant, speed_of_light,
@@ -360,19 +366,21 @@ class Xray(object):
 
 # Note: docs and function prototype are reproduced in __init__
 @require_keywords
-def xray_sld(compound, density=None,
-             wavelength=None, energy=None, natural_density=None):
+def xray_sld(compound, density=None,  natural_density=None,
+             wavelength=None, energy=None):
     """
     Compute xray scattering length densities for molecules.
 
     :Parameters:
         *compound* : Formula initializer
-            Chemical formula initializer.
+            Chemical formula.
         *density* : float | |g/cm^3|
-            Density of the compound.
-        *wavelength* : float | |Ang|
+            Mass density of the compound, or None for default.
+        *natural_density* : float | |g/cm^3|
+            Mass density of the compound at naturally occurring isotope abundance.
+        *wavelength* : float or vector | |Ang|
             Wavelength of the X-ray.
-        *energy* : float | keV
+        *energy* : float or vector | keV
             Energy of the X-ray, if *wavelength* is not specified.
 
     :Returns:
@@ -383,13 +391,9 @@ def xray_sld(compound, density=None,
         *AssertionError* :  *density* or *wavelength*/*energy* is missing.
     """
     from . import formulas
-    compound = formulas.formula(compound)
-    if density is None:
-        if natural_density is not None:
-            density = natural_density/compound.natural_mass_ratio()
-        else:
-            density = compound.density # defaults to molecule density
-    assert density is not None, "scattering calculation needs density"
+    compound = formulas.formula(compound, density=density,
+                                natural_density=natural_density)
+    assert compound.density is not None, "scattering calculation needs density"
 
     if wavelength is not None: energy = xray_energy(wavelength)
     assert energy is not None, "scattering calculation needs energy or wavelength"
@@ -405,15 +409,97 @@ def xray_sld(compound, density=None,
     if mass == 0: # because the formula is empty
         return 0,0
 
-    N = (density/mass*avogadro_number*1e-8)
+    N = (compound.density/mass*avogadro_number*1e-8)
     rho = N*sum_f1*electron_radius
     irho = N*sum_f2*electron_radius
     return rho,irho
 
+
+@require_keywords
+def index_of_refraction(compound,density=None,natural_density=None,
+                        energy=None,wavelength=None):
+    """
+    Calculates the index of refraction for a given compound
+
+    :Parameters:
+        *compound* : Formula initializer
+            Chemical formula.
+        *density* : float | |g/cm^3|
+            Mass density of the compound, or None for default.
+        *natural_density* : float | |g/cm^3|
+            Mass density of the compound at naturally occurring isotope abundance.
+        *wavelength* : float or vector | |Ang|
+            Wavelength of the X-ray.
+        *energy* : float or vector | keV
+            Energy of the X-ray, if *wavelength* is not specified.
+
+    :Returns:
+        *n* : float or vector | unitless
+            index of refraction of the material at the given energy
+
+    :Notes:
+
+    Formula taken from http://xdb.lbl.gov (section 1.7) and checked
+    against http://henke.lbl.gov/optical_constants/getdb2.html
+    """
+    if energy is not None: wavelength = xray_wavelength(energy)
+    assert wavelength is not None, "scattering calculation needs energy or wavelength"
+    f1,f2 = xray_sld(compound,
+                     density=density, natural_density=natural_density,
+                     wavelength=wavelength)
+    return 1 - wavelength**2/(2*pi)*(f1 + f2*1j)*1e-6
+
+@require_keywords
+def mirror_reflectivity(compound, density=None, natural_density=None,
+                        energy=None, wavelength=None,
+                        angle=None, roughness=0):
+    """
+    Calculates reflectivity of a thick mirror as function of energy and angle
+
+    :Parameters:
+        *compound* : Formula initializer
+            Chemical formula.
+        *density* : float | |g/cm^3|
+            Mass density of the compound, or None for default.
+        *natural_density* : float | |g/cm^3|
+            Mass density of the compound at naturally occurring isotope abundance.
+        *roughness* : float | |Ang|
+            High-spatial-frequency surface roughness.
+        *wavelength* : float or vector | |Ang|
+            Wavelength of the X-ray.
+        *energy* : float or vector | keV
+            Energy of the X-ray, if *wavelength* is not specified.
+        *angle* : vector | |deg|
+            Incident beam angles.
+
+    :Returns:
+        *reflectivity* : matrix
+            matrix of reflectivity as function of (angle,energy)
+
+    :Notes:
+
+    Formula taken from http://xdb.lbl.gov (section 4.2) and checked
+    against http://henke.lbl.gov/optical_constants/mirror2.html
+    """
+    if energy is not None: wavelength = xray_wavelength(energy)
+    assert wavelength is not None, "scattering calculation needs energy or wavelength"
+    angle = radians(angle)
+    if (numpy.isscalar(wavelength)): wavelength=numpy.array( [wavelength] )
+    if (numpy.isscalar(angle))     : angle =numpy.array( [angle] )
+    nv = index_of_refraction(compound=compound,
+                             density=density, natural_density=natural_density,
+                             wavelength=wavelength)
+    ki = 2*pi/wavelength[None,:] * sin(angle[:,None])
+    kf = 2*pi/wavelength[None,:] * sqrt(nv[None,:]**2 - cos(angle[:,None])**2)
+    r = (ki-kf)/(ki+kf)*exp(-2*ki*kf*roughness**2)
+    return abs(r)**2
+
+
 def xray_sld_from_atoms(*args, **kw):
     """
     .. deprecated:: 0.91
-        :func:`xray_sld` now accepts dictionaries of {atom: count} directly.
+
+        :func:`xray_sld` now accepts a dictionary of \{atom\: count\} directly.
     """
     return xray_sld(*args, **kw)
 
@@ -541,7 +627,7 @@ def emission_table(table=None):
     Prints a table of emission lines.
 
     :Parameters:
-        *table* : PeriodicTable.
+        *table* : PeriodicTable
             The default periodictable unless a specific table has been requested.
 
     :Returns: None
@@ -568,3 +654,4 @@ def emission_table(table=None):
     for el in table:
         if hasattr(el,'K_alpha'):
             print "%3s %7.4f %7.4f"%(el.symbol,el.K_alpha,el.K_beta1)
+
